/*
Revised simplex solver. Should be reasonably quick, may be just a little buggy.
Note: having written a simplex solver before, I just wanted something a little quicker.
This is a mess of premature optimisation. Bit-rot will no doubt set in almost immediately.
(c) Matthew Steel 2009

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.
*/


#ifndef SIMPLEX_HPP
#define SIMPLEX_HPP

#include <vector>
#include <memory>
#include <iostream>
/*
TODO:
	Deal with GE, LE constraints in construction
	Different variable bounds
	Decide what gets to be const, reference, auto_ptr...
	Decide how to do API, friendliness so B&B can work with us.
	Think about query functions, vars.
	When adding constraints we should eliminate redundancy with some sparsity.
	Put EV choice and Constraints storage into a class/classes so we can price differently
	LU factorise inverseB.
	Think about storing invVariables for artificials

NOTE:
	Was going to use Eigen. Maybe I should have. Still: Now I'm not. Nice to not have the dep,
	and nice to know precisely how the code works, but some methods are uglier for it.
*/

class Simplex
{
public:
	Simplex(std::vector<std::vector<double> > constraints, std::vector<double> costs, std::vector<double> rightHandSide);//General usage
	
	void solve(double tolerance = 1e-8);
	//Tolerance against zero for floats, probably don't need it so much.
	const std::vector<int>& getVariables() const { return variables; }
	const std::vector<unsigned>& getInvVars() const { return invVariables; }
	const std::vector<std::vector<double> >& getInvB() const { return inverseB; }
	const std::vector<double>& getX() const { return x; }
	
	double currentObjective() const { return objective; }
	bool provedInfeasible() const { return done && !optimal; }
	//Should just return objective == -infinity?
	bool isOptimal() const { return optimal; }

	//TODO: Check for dominated constraints?
	void addGEConstraint(unsigned, double);
	void addLEConstraint(unsigned, double);
	
	//TODO:
	//void addEQConstraint(unsigned, double);
	//void addGEConstraint(const std::vector<unsigned>&, const std::vector<double>&, double);
	//void addLEConstraint(const std::vector<unsigned>&, const std::vector<double>&, double);
	//void addEQConstraint(const std::vector<unsigned>&, const std::vector<double>&, double);
	
private:
	std::vector<std::vector<double> > constraints;//Column-wise
	
	std::vector<double> costs;
	
	std::vector<std::vector<double> > inverseB;//Row-wise
	std::vector<double> x;
	//Solution variables.
	
	std::vector<int> variables;
	//x[i] is the real value of the user-submitted variable variables[i].
	std::vector<unsigned> invVariables;
	//x[invVariables[i]] is the real value of the user-submitted variable i.
	
	unsigned totalVariables;
	unsigned edgeArtificial;
	double objective;
	bool optimal;
	bool done;
	
	bool realSolve(std::vector<double>&, bool, double);
	bool findEV(unsigned&, const std::vector<double>&, double);
	bool findLV(unsigned&, std::vector<double>&, unsigned, bool);
	void updateSolution(unsigned, const std::vector<double>&);
	void updateData(unsigned, unsigned, std::vector<double>&);
	
	template <class t> static void print(const std::vector<t> vec) {
		for (typename std::vector<t>::const_iterator j = vec.begin();
		j != vec.end();
		++j) {
			std::cout << *j << '\t';
		}
		std::cout << std::endl<<std::endl;
	}
	static void print(const std::vector<std::vector<double> > matrix) {
		for(std::vector<std::vector<double> >::const_iterator i = matrix.begin(); i != matrix.end(); ++i) {
			for(std::vector<double>::const_iterator j = i->begin(); j != i->end(); ++j) {
				std::cout << *j << "\t";
			}
			std::cout << std::endl;
		}
		std::cout << std::endl;
	}
	
	static double dotProduct(const std::vector<double>& left, const std::vector<double>& right)
	{
		double ret = 0.0;
		for(std::vector<double>::const_iterator i = left.begin(), j = right.begin(); j != right.end(); ++i, ++j) {
			ret += (*i)*(*j);
		}
		return ret;
	}
};

#endif
